from plugin import Plugin
from reportlab.pdfgen import canvas

class PDF(Plugin):
    def __init__(self, config):
        self.path = config.get('path')
        self.title = config.get('title')
    
    def run(self, images):
        c = canvas.Canvas(self.path)

        for image in images:
            width, height = pagesize = image.size
            c.setPageSize(pagesize)
            
            image_text = image.info['pilpe'].get('ocr')
            if image_text is not None:
                y = height - 30
                for line in image_text.split('\n'):
                    c.drawString(30, y, line)
                    y -= 15        
    
            c.drawInlineImage(image, 0, 0)
            c.showPage()

        if self.title is not None:
            c.setTitle(self.title)

        c.save()

        return images



import sys
import os.path

pwd = os.path.dirname(__file__)
if pwd:
    pilpe_path = os.path.abspath(os.path.dirname(__file__)+"/../")
else:    
    pilpe_path = os.path.abspath(os.path.dirname(__file__)+"..")
sys.path.append(pilpe_path)

from pilpe import Pipeline
from pilpe.plugins import OCR, PDF

#images_path = '%s/test/data/' % (pilpe_path)
images_path = []
images_path.append('%s/test/data/page1.tif' % (pilpe_path))
images_path.append('%s/test/data/page2.tif' % (pilpe_path))

ocr = OCR({'lang': 'por'})

pdf = PDF({
    'path': pilpe_path+'/test/pdfsaver.pdf', 
    'title': 'Projeto de lei - No. 912742', 
})

pipeline = Pipeline()
pipeline.register(ocr)
pipeline.register(pdf)
pipeline.run(images_path)



import os
import Image

class Pipeline(object):
    def __init__(self):
        self.__plugins = []
    
    def register(self, plugin):
        self.__plugins.append(plugin)
    
    def run(self, image_path):
        images = []
        if type(image_path) == list:
            for file in image_path:
                try:
                    images.append(Image.open(file))
                except Exception, e:
                    # TODO: Log warning
                    # Some files may not be valid image files
                    pass

        elif os.path.isdir(image_path):
            for file in os.listdir(image_path):
                try:
                    images.append(Image.open(image_path+'/'+file))
                except Exception, e:
                    # TODO: Log warning
                    # Some files may not be valid image files
                    pass
        
        else:
            images.append(Image.open(image_path))

        if len(images) == 0:
            return None

        [ image.info.update({'pilpe': {}}) for image in images ] 

        for plugin in self.__plugins:
            result = plugin.run(images)
            images = result
        return images  
        



import subprocess
import os
import random
from plugin import Plugin
from datetime import datetime
import tempfile

TESSERACT_BIN = 'tesseract' # Name of executable to be called at command line

class OCR(Plugin):
    def __init__(self, config):
        self.lang = config.get('lang')  

    def run(self, images):
        for image in images:
            image_text = self.__image_to_string(image)
            image.info['pilpe']['ocr'] = image_text
        return images

    def __randstr(self, lenght=8, charset='abcdefghijklmnopqrstuvxywz0123456789'):
        """Generates a random string"""
    
        password = []
        for i in range(lenght):
            rand = random.randint(0,len(charset)-1)
            password.append(charset[rand])
    
        return "-" + "".join(password)
    
    def __call_tesseract(self, scratch_image, output):
        """Calls external tesseract on input file (restrictions on types),
        outputting output+'txt'"""
        args = [TESSERACT_BIN, scratch_image, output]
        if self.lang is not None:
            args.append('-l')
            args.append(self.lang)
    
        proc = subprocess.Popen(args)
        retcode = proc.wait()
    
    def __image_to_string(self, image, cleanup = False):
        """Converts im to file, applies tesseract, and fetches resulting text.
        If cleanup=True, delete scratch files after operation."""
        
        current_sec = datetime.today().strftime("%s")
        rand = self.__randstr(8)
        scratch_image = tempfile.mkstemp('.tif')[1]
        output = tempfile.mkstemp('.txt')[1]
        
        try:
            self.__image_to_scratch(image, scratch_image)
            self.__call_tesseract(scratch_image, output[:-4])
            fp = open(output)
            txt = fp.read()
            fp.close()
        finally:
            if cleanup:
                self.__perform_cleanup(scratch_image, output)
        return txt
    
    def __image_to_scratch(self, image, scratch_image):
        """Saves image in memory to scratch file. '.tif' format will be read correctly by Tesseract"""
        image = image.convert('L')
        image.save(scratch_image, dpi=(200,200), compression=None)
         
    def __perform_cleanup(self, scratch_image, output):
        """Clean up temporary files from disk"""
        for name in (scratch_image, output+'.txt'):
            try:
                os.remove(name)
            except OSError:
                pass
    



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='pilpe',
      version='0.1',
      description='A image pipeline to apply filters in one or more images.',
      author='Sergio Campos',
      author_email='seocam@seocam.net',
      packages=find_packages(),
      package_data={'': ['*.tiff', '*.tif']},
      install_requires=[],
     )

